"use strict";(self.webpackChunkd3vis_ipynb=self.webpackChunkd3vis_ipynb||[]).push([[493],{493:(e,i,t)=>{t.r(i),t.d(i,{default:()=>r,helloWidgetPlugin:()=>d});var l=t(801),o=t(363);const d={id:"d3vis_ipynb:plugin",requires:[l.IJupyterWidgetRegistry],activate:function(e,i){i.registerWidget({name:"d3vis_ipynb",version:o.version,exports:{LinearHistPlotModel:o.LinearHistPlotModel,LinearHistPlotView:o.LinearHistPlotView,ScatterPlotModel:o.ScatterPlotModel,ScatterPlotView:o.ScatterPlotView,BarPlotModel:o.BarPlotModel,BarPlotView:o.BarPlotView,HistogramPlotModel:o.HistogramPlotModel,HistogramPlotView:o.HistogramPlotView,EmbeddingModel:o.EmbeddingModel,EmbeddingView:o.EmbeddingView,RangeSliderModel:o.RangeSliderModel,RangeSliderView:o.RangeSliderView}})},autoStart:!0},r=d}}]);